#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright 2012-2015 Matt Martz
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import re
import sys
import math
import signal
import socket
import timeit
import platform
import threading

__version__ = '0.3.4'

# Some global variables we use
user_agent = None
source = None
shutdown_event = None
scheme = 'http'


# Used for bound_interface
socket_socket = socket.socket

try:
    import xml.etree.cElementTree as ET
except ImportError:
    try:
        import xml.etree.ElementTree as ET
    except ImportError:
        from xml.dom import minidom as DOM
        ET = None

# Begin import game to handle Python 2 and Python 3
try:
    from urllib2 import urlopen, Request, HTTPError, URLError
except ImportError:
    from urllib.request import urlopen, Request, HTTPError, URLError

try:
    from httplib import HTTPConnection, HTTPSConnection
except ImportError:
    e_http_py2 = sys.exc_info()
    try:
        from http.client import HTTPConnection, HTTPSConnection
    except ImportError:
        e_http_py3 = sys.exc_info()
        raise SystemExit('Your python installation is missing required HTTP '
                         'client classes:\n\n'
                         'Python 2: %s\n'
                         'Python 3: %s' % (e_http_py2[1], e_http_py3[1]))

try:
    from Queue import Queue
except ImportError:
    from queue import Queue

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

try:
    from urlparse import parse_qs
except ImportError:
    try:
        from urllib.parse import parse_qs
    except ImportError:
        from cgi import parse_qs

try:
    from hashlib import md5
except ImportError:
    from md5 import md5

try:
    from argparse import ArgumentParser as ArgParser
except ImportError:
    from optparse import OptionParser as ArgParser

# ljjm: silence printing
silence=True
if silence:
    def noprint(*args, **kwargs):
        pass
    print_ = noprint
else:
    try:
        import builtins
    except ImportError:
        def print_(*args, **kwargs):
            """The new-style print function taken from
            https://pypi.python.org/pypi/six/

            """
            fp = kwargs.pop("file", sys.stdout)
            if fp is None:
                return

            def write(data):
                if not isinstance(data, basestring):
                    data = str(data)
                fp.write(data)

            want_unicode = False
            sep = kwargs.pop("sep", None)
            if sep is not None:
                if isinstance(sep, unicode):
                    want_unicode = True
                elif not isinstance(sep, str):
                    raise TypeError("sep must be None or a string")
            end = kwargs.pop("end", None)
            if end is not None:
                if isinstance(end, unicode):
                    want_unicode = True
                elif not isinstance(end, str):
                    raise TypeError("end must be None or a string")
            if kwargs:
                raise TypeError("invalid keyword arguments to print()")
            if not want_unicode:
                for arg in args:
                    if isinstance(arg, unicode):
                        want_unicode = True
                        break
            if want_unicode:
                newline = unicode("\n")
                space = unicode(" ")
            else:
                newline = "\n"
                space = " "
            if sep is None:
                sep = space
            if end is None:
                end = newline
            for i, arg in enumerate(args):
                if i:
                    write(sep)
                write(arg)
            write(end)
    else:
        print_ = getattr(builtins, 'print')
        del builtins


class SpeedtestCliServerListError(Exception):
    """Internal Exception class used to indicate to move on to the next
    URL for retrieving speedtest.net server details

    """


def bound_socket(*args, **kwargs):
    """Bind socket to a specified source IP address"""

    global source
    sock = socket_socket(*args, **kwargs)
    sock.bind((source, 0))
    return sock


def distance(origin, destination):
    """Determine distance between 2 sets of [lat,lon] in km"""

    lat1, lon1 = origin
    lat2, lon2 = destination
    radius = 6371  # km

    dlat = math.radians(lat2 - lat1)
    dlon = math.radians(lon2 - lon1)
    a = (math.sin(dlat / 2) * math.sin(dlat / 2) +
         math.cos(math.radians(lat1)) *
         math.cos(math.radians(lat2)) * math.sin(dlon / 2) *
         math.sin(dlon / 2))
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
    d = radius * c

    return d


def build_user_agent():
    """Build a Mozilla/5.0 compatible User-Agent string"""

    global user_agent
    if user_agent:
        return user_agent

    ua_tuple = (
        'Mozilla/5.0',
        '(%s; U; %s; en-us)' % (platform.system(), platform.architecture()[0]),
        'Python/%s' % platform.python_version(),
        '(KHTML, like Gecko)',
        'speedtest-cli/%s' % __version__
    )
    user_agent = ' '.join(ua_tuple)
    return user_agent


def build_request(url, data=None, headers={}):
    """Build a urllib2 request object

    This function automatically adds a User-Agent header to all requests

    """

    if url[0] == ':':
        schemed_url = '%s%s' % (scheme, url)
    else:
        schemed_url = url

    headers['User-Agent'] = user_agent
    return Request(schemed_url, data=data, headers=headers)


def catch_request(request):
    """Helper function to catch common exceptions encountered when
    establishing a connection with a HTTP/HTTPS request

    """

    try:
        uh = urlopen(request)
        return uh, False
    except (HTTPError, URLError, socket.error):
        e = sys.exc_info()[1]
        return None, e


class FileGetter(threading.Thread):
    """Thread class for retrieving a URL"""

    def __init__(self, url, start):
        self.url = url
        self.result = None
        self.starttime = start
        threading.Thread.__init__(self)

    def run(self):
        self.result = [0]
        try:
            if (timeit.default_timer() - self.starttime) <= 10:
                request = build_request(self.url)
                f = urlopen(request)
                while 1 and not shutdown_event.isSet():
                    self.result.append(len(f.read(10240)))
                    if self.result[-1] == 0:
                        break
                f.close()
        except IOError:
            pass


def downloadSpeed(files, quiet=False):
    """Function to launch FileGetter threads and calculate download speeds"""

    start = timeit.default_timer()

    def producer(q, files):
        for file in files:
            thread = FileGetter(file, start)
            thread.start()
            q.put(thread, True)
            #if not quiet and not shutdown_event.isSet():
                #sys.stdout.write('.')
                #sys.stdout.flush()

    finished = []

    def consumer(q, total_files):
        while len(finished) < total_files:
            thread = q.get(True)
            while thread.isAlive():
                thread.join(timeout=0.1)
            finished.append(sum(thread.result))
            del thread

    q = Queue(6)
    prod_thread = threading.Thread(target=producer, args=(q, files))
    cons_thread = threading.Thread(target=consumer, args=(q, len(files)))
    start = timeit.default_timer()
    prod_thread.start()
    cons_thread.start()
    while prod_thread.isAlive():
        prod_thread.join(timeout=0.1)
    while cons_thread.isAlive():
        cons_thread.join(timeout=0.1)
    return (sum(finished) / (timeit.default_timer() - start))


class FilePutter(threading.Thread):
    """Thread class for putting a URL"""

    def __init__(self, url, start, size):
        self.url = url
        chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'
        data = chars * (int(round(int(size) / 36.0)))
        self.data = ('content1=%s' % data[0:int(size) - 9]).encode()
        del data
        self.result = None
        self.starttime = start
        threading.Thread.__init__(self)

    def run(self):
        try:
            if ((timeit.default_timer() - self.starttime) <= 10 and
                    not shutdown_event.isSet()):
                request = build_request(self.url, data=self.data)
                f = urlopen(request)
                f.read(11)
                f.close()
                self.result = len(self.data)
            else:
                self.result = 0
        except IOError:
            self.result = 0


def uploadSpeed(url, sizes, quiet=False):
    """Function to launch FilePutter threads and calculate upload speeds"""

    start = timeit.default_timer()

    def producer(q, sizes):
        for size in sizes:
            thread = FilePutter(url, start, size)
            thread.start()
            q.put(thread, True)
            #if not quiet and not shutdown_event.isSet():
                #sys.stdout.write('.')
                #sys.stdout.flush()

    finished = []

    def consumer(q, total_sizes):
        while len(finished) < total_sizes:
            thread = q.get(True)
            while thread.isAlive():
                thread.join(timeout=0.1)
            finished.append(thread.result)
            del thread

    q = Queue(6)
    prod_thread = threading.Thread(target=producer, args=(q, sizes))
    cons_thread = threading.Thread(target=consumer, args=(q, len(sizes)))
    start = timeit.default_timer()
    prod_thread.start()
    cons_thread.start()
    while prod_thread.isAlive():
        prod_thread.join(timeout=0.1)
    while cons_thread.isAlive():
        cons_thread.join(timeout=0.1)
    return (sum(finished) / (timeit.default_timer() - start))


def getAttributesByTagName(dom, tagName):
    """Retrieve an attribute from an XML document and return it in a
    consistent format

    Only used with xml.dom.minidom, which is likely only to be used
    with python versions older than 2.5
    """
    elem = dom.getElementsByTagName(tagName)[0]
    return dict(list(elem.attributes.items()))


def getConfig():
    """Download the speedtest.net configuration and return only the data
    we are interested in
    """

    request = build_request('://www.speedtest.net/speedtest-config.php')
    uh, e = catch_request(request)
    if e:
        print_('Could not retrieve speedtest.net configuration: %s' % e)
        sys.exit(1)
    configxml = []
    while 1:
        configxml.append(uh.read(10240))
        if len(configxml[-1]) == 0:
            break
    if int(uh.code) != 200:
        return None
    uh.close()
    try:
        try:
            root = ET.fromstring(''.encode().join(configxml))
            config = {
                'client': root.find('client').attrib,
                'times': root.find('times').attrib,
                'download': root.find('download').attrib,
                'upload': root.find('upload').attrib}
        except AttributeError:  # Python3 branch
            root = DOM.parseString(''.join(configxml))
            config = {
                'client': getAttributesByTagName(root, 'client'),
                'times': getAttributesByTagName(root, 'times'),
                'download': getAttributesByTagName(root, 'download'),
                'upload': getAttributesByTagName(root, 'upload')}
    except SyntaxError:
        print_('Failed to parse speedtest.net configuration')
        sys.exit(1)
    del root
    del configxml
    return config


def closestServers(client, all=False):
    """Determine the 5 closest speedtest.net servers based on geographic
    distance
    """

    urls = [
        '://www.speedtest.net/speedtest-servers-static.php',
        '://c.speedtest.net/speedtest-servers-static.php',
        '://www.speedtest.net/speedtest-servers.php',
        '://c.speedtest.net/speedtest-servers.php',
    ]
    errors = []
    servers = {}
    for url in urls:
        try:
            request = build_request(url)
            uh, e = catch_request(request)
            if e:
                errors.append('%s' % e)
                raise SpeedtestCliServerListError
            serversxml = []
            while 1:
                serversxml.append(uh.read(10240))
                if len(serversxml[-1]) == 0:
                    break
            if int(uh.code) != 200:
                uh.close()
                raise SpeedtestCliServerListError
            uh.close()
            try:
                try:
                    root = ET.fromstring(''.encode().join(serversxml))
                    elements = root.getiterator('server')
                except AttributeError:  # Python3 branch
                    root = DOM.parseString(''.join(serversxml))
                    elements = root.getElementsByTagName('server')
            except SyntaxError:
                raise SpeedtestCliServerListError
            for server in elements:
                try:
                    attrib = server.attrib
                except AttributeError:
                    attrib = dict(list(server.attributes.items()))
                d = distance([float(client['lat']),
                              float(client['lon'])],
                             [float(attrib.get('lat')),
                              float(attrib.get('lon'))])
                attrib['d'] = d
                if d not in servers:
                    servers[d] = [attrib]
                else:
                    servers[d].append(attrib)
            del root
            del serversxml
            del elements
        except SpeedtestCliServerListError:
            continue

        # We were able to fetch and parse the list of speedtest.net servers
        if servers:
            break

    if not servers:
        print_('Failed to retrieve list of speedtest.net servers:\n\n %s' %
               '\n'.join(errors))
        sys.exit(1)

    closest = []
    for d in sorted(servers.keys()):
        for s in servers[d]:
            closest.append(s)
            if len(closest) == 5 and not all:
                break
        else:
            continue
        break

    del servers
    return closest


def getBestServer(servers):
    """Perform a speedtest.net latency request to determine which
    speedtest.net server has the lowest latency
    """

    results = {}
    for server in servers:
        cum = []
        url = '%s/latency.txt' % os.path.dirname(server['url'])
        urlparts = urlparse(url)
        for i in range(0, 3):
            try:
                if urlparts[0] == 'https':
                    h = HTTPSConnection(urlparts[1])
                else:
                    h = HTTPConnection(urlparts[1])
                headers = {'User-Agent': user_agent}
                start = timeit.default_timer()
                h.request("GET", urlparts[2], headers=headers)
                r = h.getresponse()
                total = (timeit.default_timer() - start)
            except (HTTPError, URLError, socket.error):
                cum.append(3600)
                continue
            text = r.read(9)
            if int(r.status) == 200 and text == 'test=test'.encode():
                cum.append(total)
            else:
                cum.append(3600)
            h.close()
        avg = round((sum(cum) / 6) * 1000, 3)
        results[avg] = server
    fastest = sorted(results.keys())[0]
    best = results[fastest]
    best['latency'] = fastest

    return best


def ctrl_c(signum, frame):
    """Catch Ctrl-C key sequence and set a shutdown_event for our threaded
    operations
    """

    global shutdown_event
    shutdown_event.set()
    raise SystemExit('\nCancelling...')


def version():
    """Print the version"""

    raise SystemExit(__version__)


def speedtest():
    """Run the full speedtest.net test"""

    global shutdown_event, source, scheme
    shutdown_event = threading.Event()

    signal.signal(signal.SIGINT, ctrl_c)

    description = (
        'Command line interface for testing internet bandwidth using '
        'speedtest.net.\n'
        '------------------------------------------------------------'
        '--------------\n'
        'https://github.com/sivel/speedtest-cli')

    parser = ArgParser(description=description)
    # Give optparse.OptionParser an `add_argument` method for
    # compatibility with argparse.ArgumentParser
    try:
        parser.add_argument = parser.add_option
    except AttributeError:
        pass
    parser.add_argument('--bytes', dest='units', action='store_const',
                        const=('byte', 1), default=('bit', 8),
                        help='Display values in bytes instead of bits. Does '
                             'not affect the image generated by --share')
    parser.add_argument('--share', action='store_true',
                        help='Generate and provide a URL to the speedtest.net '
                             'share results image')
    parser.add_argument('--simple', action='store_true',
                        help='Suppress verbose output, only show basic '
                             'information')
    parser.add_argument('--list', action='store_true',
                        help='Display a list of speedtest.net servers '
                             'sorted by distance')
    parser.add_argument('--server', help='Specify a server ID to test against')
    parser.add_argument('--mini', help='URL of the Speedtest Mini server')
    parser.add_argument('--source', help='Source IP address to bind to')
    parser.add_argument('--timeout', default=10, type=int,
                        help='HTTP timeout in seconds. Default 10')
    parser.add_argument('--secure', action='store_true',
                        help='Use HTTPS instead of HTTP when communicating '
                             'with speedtest.net operated servers')
    parser.add_argument('--version', action='store_true',
                        help='Show the version number and exit')

    # ljmm@sics.se hack:
    parseARGV=False
    if parseARGV:
        options = parser.parse_args()
    else:
        options = parser.parse_args([])

    if isinstance(options, tuple):
        args = options[0]
    else:
        args = options
    del options

    # Print the version and exit
    if args.version:
        version()

    socket.setdefaulttimeout(args.timeout)

    # Pre-cache the user agent string
    build_user_agent()

    # If specified bind to a specific IP address
    if args.source:
        source = args.source
        socket.socket = bound_socket

    if args.secure:
        scheme = 'https'

    if not args.simple:
        print_('Retrieving speedtest.net configuration...')
    try:
        config = getConfig()
    except URLError:
        print_('Cannot retrieve speedtest configuration')
        sys.exit(1)

    if not args.simple:
        print_('Retrieving speedtest.net server list...')
    if args.list or args.server:
        servers = closestServers(config['client'], True)
        if args.list:
            serverList = []
            for server in servers:
                line = ('%(id)4s) %(sponsor)s (%(name)s, %(country)s) '
                        '[%(d)0.2f km]' % server)
                serverList.append(line)
            print_('\n'.join(serverList).encode('utf-8', 'ignore'))
            sys.exit(0)
    else:
        servers = closestServers(config['client'])

    if not args.simple:
        print_('Testing from %(isp)s (%(ip)s)...' % config['client'])

    if args.server:
        try:
            best = getBestServer(filter(lambda x: x['id'] == args.server,
                                        servers))
        except IndexError:
            print_('Invalid server ID')
            sys.exit(1)
    elif args.mini:
        name, ext = os.path.splitext(args.mini)
        if ext:
            url = os.path.dirname(args.mini)
        else:
            url = args.mini
        urlparts = urlparse(url)
        try:
            request = build_request(args.mini)
            f = urlopen(request)
        except:
            print_('Invalid Speedtest Mini URL')
            sys.exit(1)
        else:
            text = f.read()
            f.close()
        extension = re.findall('upload_extension: "([^"]+)"', text.decode())
        if not extension:
            for ext in ['php', 'asp', 'aspx', 'jsp']:
                try:
                    request = build_request('%s/speedtest/upload.%s' %
                                            (args.mini, ext))
                    f = urlopen(request)
                except:
                    pass
                else:
                    data = f.read().strip()
                    if (f.code == 200 and
                            len(data.splitlines()) == 1 and
                            re.match('size=[0-9]', data)):
                        extension = [ext]
                        break
        if not urlparts or not extension:
            print_('Please provide the full URL of your Speedtest Mini server')
            sys.exit(1)
        servers = [{
            'sponsor': 'Speedtest Mini',
            'name': urlparts[1],
            'd': 0,
            'url': '%s/speedtest/upload.%s' % (url.rstrip('/'), extension[0]),
            'latency': 0,
            'id': 0
        }]
        try:
            best = getBestServer(servers)
        except:
            best = servers[0]
    else:
        if not args.simple:
            print_('Selecting best server based on latency...')
        best = getBestServer(servers)

    if not args.simple:
        print_(('Hosted by %(sponsor)s (%(name)s) [%(d)0.2f km]: '
               '%(latency)s ms' % best).encode('utf-8', 'ignore'))
    else:
        print_('Ping: %(latency)s ms' % best)

    sizes = [350, 500, 750, 1000, 1500, 2000, 2500, 3000, 3500, 4000]
    urls = []
    for size in sizes:
        for i in range(0, 4):
            urls.append('%s/random%sx%s.jpg' %
                        (os.path.dirname(best['url']), size, size))
    if not args.simple:
        print_('Testing download speed', end='')
    dlspeed = downloadSpeed(urls, args.simple)
    if not args.simple:
        print_()
    print_('Download: %0.2f M%s/s' %
           ((dlspeed / 1000 / 1000) * args.units[1], args.units[0]))

    sizesizes = [int(.25 * 1000 * 1000), int(.5 * 1000 * 1000)]
    sizes = []
    for size in sizesizes:
        for i in range(0, 25):
            sizes.append(size)
    if not args.simple:
        print_('Testing upload speed', end='')
    ulspeed = uploadSpeed(best['url'], sizes, args.simple)
    if not args.simple:
        print_()
    print_('Upload: %0.2f M%s/s' %
           ((ulspeed / 1000 / 1000) * args.units[1], args.units[0]))

    if args.share and args.mini:
        print_('Cannot generate a speedtest.net share results image while '
               'testing against a Speedtest Mini server')
    elif args.share:
        dlspeedk = int(round((dlspeed / 1000) * 8, 0))
        ping = int(round(best['latency'], 0))
        ulspeedk = int(round((ulspeed / 1000) * 8, 0))

        # Build the request to send results back to speedtest.net
        # We use a list instead of a dict because the API expects parameters
        # in a certain order
        apiData = [
            'download=%s' % dlspeedk,
            'ping=%s' % ping,
            'upload=%s' % ulspeedk,
            'promo=',
            'startmode=%s' % 'pingselect',
            'recommendedserverid=%s' % best['id'],
            'accuracy=%s' % 1,
            'serverid=%s' % best['id'],
            'hash=%s' % md5(('%s-%s-%s-%s' %
                             (ping, ulspeedk, dlspeedk, '297aae72'))
                            .encode()).hexdigest()]

        headers = {'Referer': 'http://c.speedtest.net/flash/speedtest.swf'}
        request = build_request('://www.speedtest.net/api/api.php',
                                data='&'.join(apiData).encode(),
                                headers=headers)
        f, e = catch_request(request)
        if e:
            print_('Could not submit results to speedtest.net: %s' % e)
            sys.exit(1)
        response = f.read()
        code = f.code
        f.close()

        if int(code) != 200:
            print_('Could not submit results to speedtest.net')
            sys.exit(1)

        qsargs = parse_qs(response.decode())
        resultid = qsargs.get('resultid')
        if not resultid or len(resultid) != 1:
            print_('Could not submit results to speedtest.net')
            sys.exit(1)

        print_('Share results: %s://www.speedtest.net/result/%s.png' %
               (scheme, resultid[0]))
    data = {
        'download': dlspeed,
        'ping'    : best['latency'],
        'upload'  : ulspeed,
        'serverid': best['id'],
    }
    return data

def main():
    try:
        speedtest()
    except KeyboardInterrupt:
        print_('\nCancelling...')


if __name__ == '__main__':
    main()

# vim:ts=4:sw=4:expandtab



import time
import os
import re
import logging
import socket

import Task
import cheesepi as cp

logger = cp.config.get_logger(__name__)

class Httping(Task.Task):

	# construct the process and perform pre-work
	def __init__(self, dao, parameters):
		Task.Task.__init__(self, dao, parameters)
		self.spec['taskname'] = "httping"
		if not 'landmark'    in self.spec: self.spec['landmark']    = "www.sics.se"
		if not 'ping_count'  in self.spec: self.spec['ping_count']  = 10
		try:
			socket.gethostbyname(self.spec['landmark']) # we dont care, just populate the cache
		except:
			pass # record network failure later...

	def toDict(self):
		return self.spec

	# actually perform the measurements, no arguments required
	def run(self):
		logger.info("HTTPinging: %s @ %f, PID: %d" % (self.spec['landmark'], time.time(), os.getpid()))
		self.measure(self.spec['landmark'], self.spec['ping_count'])


	#main measure funtion
	def measure(self, landmark, ping_count):
		start_time = cp.utils.now()
		op_output = self.perform(landmark, ping_count)
		end_time = cp.utils.now()
		logger.debug(op_output)
		if op_output!=None:
			self.parse_output(op_output, landmark, start_time, end_time, ping_count)
		self.dao.write_op("httping", self.spec)

	#ping function
	def perform(self, landmark, ping_count):
		command = "httping -S -c %s %s" % (ping_count, landmark)
		logging.info("Executing: "+command)
		self.spec['return_code'], output = self.execute(command)
		#print self.spec['return_code'], output
		if self.spec['return_code']==0:
			return output
		return None

	# average out the breakdowns if different steps in HTTP request
	def parse_breakdowns(self, breakdowns):
		acc=[0.0] * len(breakdowns[0])
		for i in xrange(len(breakdowns)):
			for j in xrange(len(breakdowns[i])):
				acc[j] += float(breakdowns[i][j])
		return [x / len(breakdowns) for x in acc]

	#read the data from ping and reformat for database entry
	def parse_output(self, data, landmark, start_time, end_time, ping_count):
		self.spec["landmark"]    = landmark
		self.spec["start_time"]  = start_time
		self.spec["end_time"]    = end_time
		self.spec["ping_count"]  = int(ping_count)
		delays=[]
		breakdowns=[]
		downloaded = 0 # how many bytes delivered?

		lines = data.split("\n")
		first_line = lines.pop(0).split()
		self.spec["destination_domain"]  = first_line[1]

		delays = [-1.0] * ping_count# initialise storage
		for line in lines:
			if "time=" in line: # is this a PING return line?
				# does the following string wrangling always hold? what if not "X ms" ?
				# also need to check whether we are on linux-like or BSD-like ping
				downloaded  += int(re.findall('[\d]+ bytes',line)[0][:-6])
				sequence_num = int(re.findall('seq=[\d]+ ',line)[0][4:-1])

				delay = re.findall('[\d\.]+ ms',line)[0][0:-3]
				# only save returned pings!
				delays[sequence_num]=float(delay)

				# capture split breakdown of httping
				splits = re.findall('time=.*=',line)[0][5:-1]
				breakdowns.append(splits.split("+"))

			if "packet loss" in line:
				loss = re.findall('[\d]+% packet loss',line)[0][:-13]
				self.spec["packet_loss"] = float(loss)
			elif "min/avg/max" in line:
				fields = line.split()[3].split("/")
				self.spec["minimum_RTT"] = float(fields[0])
				self.spec["average_RTT"] = float(fields[1])
				self.spec["maximum_RTT"] = float(fields[2])
		self.spec['delays']     = str(delays)
		self.spec["stddev_RTT"] = cp.utils.stdev(delays)
		self.spec['breakdown']  = str(self.parse_breakdowns(breakdowns))
		self.spec['downloaded'] = downloaded


if __name__ == "__main__":
	#general logging here? unable to connect etc
	dao    = cp.config.get_dao()
	config = cp.config.get_config()

	spec = {'landmark':'www.sics.se'}
	httping_task = Httping(dao, spec)
	httping_task.run()







import dao

try:
	import dao_mongo
except ImportError as e:
	print "Missing Mongo python module (and GridFS and bson), use 'pip install pymongo'"
	print str(e)

try:
	import dao_influx08
except ImportError as e:
	print "Missing InfluxDB python module, use 'pip install influxdb'"
	print str(e)

try:
	import dao_influx09
except ImportError as e:
	print "Missing InfluxDB python module, use 'pip install influxdb'"
	print str(e)

#try:
#	import dao_mysql
#except ImportError as e:
#	print "Missing MySQL python module, use 'pip install MySQL-python'"
#	print str(e)




import time
import os

# http://www.dnspython.org/examples.html
import dns.query
import dns.resolver
from dns.exception import DNSException

import cheesepi as cp
import Task

logger = cp.config.get_logger(__name__)

class DNS(Task.Task):

	# construct the process and perform pre-work
	def __init__(self, dao, spec):
		Task.Task.__init__(self, dao, spec)
		self.spec['taskname'] = "dns"
		if not 'landmark' in spec: self.spec['landmark'] = "sics.se"

	# actually perform the measurements, no arguments required
	def run(self):
		logger.info("DNS: @ %f, PID: %d" % (time.time(), os.getpid()))
		self.measure()

	# measure and record funtion
	def measure(self):
		try:
			op_output = self.query_authoritative_ns(self.spec['landmark'], log)
			print op_output
		except Exception as e:
			logger.error("DNS query failed: %s" % e)
			return
		logger.debug(op_output)
		parsed_output = self.parse_output(op_output, self.spec['landmark'])
		self.dao.write_op(self.spec['taskname'], parsed_output)

	#read the data from ping and reformat for database entry
	def parse_output(self, delays, domain):
		self.spec['delays'] = str(delays)
		self.spec['sum']    = sum(delays)
		return self.spec

	# http://stackoverflow.com/questions/4066614/how-can-i-find-the-authoritative-dns-server-for-a-domain-using-dnspython
	def query_authoritative_ns(self, domain, log=lambda msg: None):
		default = dns.resolver.get_default_resolver()
		ns = default.nameservers[0]
		n = domain.split('.')
		delays = [] # right to left delays in DNS resolution

		for i in xrange(len(n), 0, -1):
			sub = '.'.join(n[i-1:])
			log('Looking up %s on %s' % (sub, ns))
			start_time = cp.utils.now()
			query = dns.message.make_query(sub, dns.rdatatype.NS)
			response = dns.query.udp(query, ns)
			end_time = cp.utils.now()
			delay = end_time - start_time
			logger.info("time: %f" % (delay))
			delays.append(delay)

			rcode = response.rcode()
			if rcode != dns.rcode.NOERROR:
				if rcode == dns.rcode.NXDOMAIN:
					raise Exception('%s does not exist.' % (sub))
				else:
					raise Exception('Error %s' % (dns.rcode.to_text(rcode)))

			if len(response.authority) > 0:
				rrsets = response.authority
			elif len(response.additional) > 0:
				rrsets = [response.additional]
			else:
				rrsets = response.answer

			# Handle all RRsets, not just the first one
			for rrset in rrsets:
				#print rrset
				for rr in rrset:
					if rr.rdtype == dns.rdatatype.SOA:
						#log('Same server is authoritative for %s' % (sub))
						pass
					elif rr.rdtype == dns.rdatatype.A:
						ns = rr.items[0].address
						#log('Glue record for %s: %s' % (rr.name, ns))
					elif rr.rdtype == dns.rdatatype.NS:
						authority = rr.target
						ns = default.query(authority).rrset[0].to_text()
						#log('%s [%s] is authoritative for %s; ttl %i' % (authority, ns, sub, rrset.ttl))
						result = rrset
					else:
						# IPv6 glue records etc
						#log('Ignoring %s' % (rr))
						pass
		#return result
		return delays


def log (msg):
	pass
	#sys.stderr.write(msg + u'\n')


if __name__ == "__main__":
	#general logging here? unable to connect etc
	dao = cp.config.get_dao()

	spec = {'landmark':"www.abc.net.au"}
	dns_task = DNS(dao, spec)
	dns_task.run()




""" Copyright (c) 2015, Swedish Institute of Computer Science
  All rights reserved.
  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
	  * Redistributions of source code must retain the above copyright
		notice, this list of conditions and the following disclaimer.
	  * Redistributions in binary form must reproduce the above copyright
		notice, this list of conditions and the following disclaimer in the
		documentation and/or other materials provided with the distribution.
	  * Neither the name of The Swedish Institute of Computer Science nor the
		names of its contributors may be used to endorse or promote products
		derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL THE SWEDISH INSTITUTE OF COMPUTER SCIENCE BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Authors: ljjm@sics.se
Testers:
"""

import os
import sys
import json
import urllib2
import uuid
import time
import md5
import argparse
import multiprocessing
import platform
import netifaces
from subprocess import call

import cheesepi as cp
from cheesepi.tasks import *

logger = cp.config.get_logger(__name__)


def console_script():
	"""Command line tool, installed through setup.py"""
	commands = ['start','stop','status','reset','upgrade']
	options  = ['dispatcher','storage','influxdb','dashboard','all']

	parser = argparse.ArgumentParser(prog='cheesepi')
	parser.add_argument('command', metavar='COMMAND', choices=commands, nargs='?',
		help="'start' or 'stop' one of the CheesePi components.\n"+
			"status, reset or upgrade")
	parser.add_argument('option',  metavar='OPTION', choices=options, nargs='?',
		help='Options to the command. If starting or stopping, then name the component'+
			'storage, dispatcher or dashboard')
	args = parser.parse_args()

	# single parameter commands
	if   args.command=="status":  show_status()
	elif args.command=="reset":   reset_install()
	elif args.command=="upgrade": upgrade_install()
	elif args.command=="list": list_data(args.option)
	else: # both command and option required
		if   args.option=='dispatcher': control_dispatcher(args.command)
		elif args.option=='storage' or args.option=='influxdb': control_storage(args.command)
		elif args.option=='dashboard':  control_dashboard(args.command)
		elif args.option=='all':        control_all(args.command)
		else:
			print "Error: unknown OPTION: %s" % args.option


def show_status():
	"""Just print the location of important CheesePi dirs/files"""
	schedule_file = os.path.join(cp.config.cheesepi_dir, cp.config.get('schedule'))
	print "Status of CheesePi install (version %s):" % cp.config.version()
	print "Install dir:\t%s"   % cp.config.cheesepi_dir
	print "Log file:\t%s"      % cp.config.log_file
	print "Config file:\t%s"   % cp.config.config_file
	print "Schedule file:\t%s" % schedule_file

def list_data(task="ping"):
	dao = cp.config.get_dao()
	print dao.read_op(task)

def control_dispatcher(action):
	"""Start or stop the dispatcher"""
	if action=='start':
		print "Starting the dispatcher..."
		cp.dispatcher.start()
	else:
		print "Error: action not yet implemented!"

def copy_influx_config(influx_config):
	"""Copy the default influx config to a local copt (probably in $HOME)"""
	print "Warning: making local config: %s" % influx_config
	storage_dir = "/var/lib/influxdb"
	if not os.path.exists(storage_dir):
		print "Warning: Default InfluxDB storage dir %s does not exist!" % storage_dir
	influx_dir = cp.config.cheesepi_dir+"/bin/tools/influxdb"
	default_config  = os.path.join(influx_dir,"config.toml")
	print "Warning: copying from default config: %s" % default_config
	cp.config.copyfile(default_config, influx_config, replace={"INFLUX_DIR":influx_dir} )

def test_execute(cmd_array):
	"""Return true if cmd_array can execute"""
	try:
		call(cmd_array)
		return True
	except:
		return False

def find_influx_exe():
	"""See which influxdb we should use"""
	config_path = cp.config.get("database_exe")
	if config_path: # if database exectable was set in config file
		print "set it configfile"
		return config_path
	elif (test_execute(["influxdb","-h"])):
		print "system influx"
		return "influxdb"
	else:
		if isARM():
			print "cheesepi influx"
			return cp.config.cheesepi_dir+"/bin/tools/influxdb/influxdb.arm"
		else:
			print "Error: Can't find InfluxDB binary, set 'database_exe' in the config file"
			sys.exit(1)

def control_storage(action):
	"""Start or stop InfluxDB, either the bundled or the system version"""
	storage_dir = "/var/lib/influxdb"
	if not os.path.exists(storage_dir):
		print "Warning: Default InfluxDB storage dir %s does not exist!" % storage_dir

	if action=='start':
		print "Starting InfluxDB..."
		home_dir = os.path.expanduser("~")
		influx_config = os.path.join(home_dir,".influxconfig.toml")
		# test if we have already made the local config file
		if not os.path.isfile(influx_config):
			copy_influx_config(influx_config)
		influx=find_influx_exe()
		# start the influx server
		print "Running: "+influx+" -config="+influx_config
		try:
			call([influx, "-config="+influx_config])
		except Exception as e:
			msg = "Problem executing influxdb command %s -config=%s: %s" % (influx,influx_config,e)
			msg += "\nCheck PATH inclusion of system and python 'bin' directories"
			print msg
			logger.error(msg)
	else:
		print "Error: action not yet implemented!"

def control_dashboard(action):
	"""Start or stop the webserver that hosts the dashboard"""
	if action=='start':
		print "Starting the dashboard..."
		cp.bin.webserver.webserver.setup_server()
	else:
		print "Error: action not yet implemented!"

def control_all(action):
	pool = multiprocessing.Pool(processes=3)
	pool.apply_async(control_storage,  [action])
	time.sleep(3) # allow spoolup and config generation
	pool.apply_async(control_dispatcher,[action])
	pool.apply_async(control_dashboard, [action])
	pool.close()
	pool.join()

def reset_install():
	"""Wipe all local changes to schedule, config"""
	home_dir = os.path.expanduser("~")
	influx_config = os.path.join(home_dir,".influxconfig.toml")
	copy_influx_config(influx_config)
	cp.config.ensure_default_config(True)

def upgrade_install():
	"""Try and pull new version of the code from PyPi"""
	upgrade_task = cp.tasks.Upgradecode()
	upgrade_task.run()




def make_series():
	"""Ensure that database contains series grafana and cheesepi"""
	dao = cp.config.get_dao()
	# TODO: dont make if already exists
	dao.make_database("cheesepi")
	dao.make_database("grafana")

def build_json(dao, json_str):
	"""Build a Task object out of a JSON string spec"""
	spec = json.loads(json_str)
	return build_task(dao, spec)

def build_task(dao, spec):
	if not 'taskname' in spec:
		logger.error("No 'taskname' specified!")
		return None

	spec['taskname'] = spec['taskname'].lower()
	if spec['taskname']=='ping':
		return Ping(dao, spec)
	elif spec['taskname']=='httping':
		return Httping(dao, spec)
	elif spec['taskname']=='traceroute':
		return Traceroute(dao, spec)
	elif spec['taskname']=='dash':
		return Dash(dao, spec)
	elif spec['taskname']=='dns':
		return DNS(dao, spec)
	elif spec['taskname']=='throughput':
		return Throughput(dao, spec)
	elif spec['taskname']=='iperf':
		return iPerf(dao, spec)
	elif spec['taskname']=='beacon':
		return Beacon(dao, spec)
	elif spec['taskname']=='upload':
		return Upload(dao, spec)
	elif spec['taskname']=='status':
		return Status(dao, spec)
	elif spec['taskname']=='wifi':
		return Wifi(dao, spec)
	elif spec['taskname']=='dummy':
		return Dummy(dao, spec)
	elif spec['taskname']=='upload':
		return Upload(dao, spec)
	elif spec['taskname']=='upgradecode':
		return Upgradecode(dao, spec)
	elif spec['taskname']=='updatetasks':
		return Updatetasks(dao, spec)
	else:
		raise Exception('Task name not specified! '+str(spec))


# time functions
def now():
	return time.time()
	#return int(datetime.datetime.utcnow().strftime("%s"))

def isARM():
	if "arm" in platform.machine():
		return True
	return False

# logging facilities
def write_file(ret, start_time, ethmac):
	filename = "./"+ethmac+str(start_time)+".txt"
	fd = open(filename, 'w')
	fd.write(ret)
	fd.close()


def get_MAC():
	"""Return the MAC of this device's first NIC"""
	return str(hex(uuid.getnode()))[2:]

def get_host_id():
	"""Return this host's ID"""
	return str(md5.new(get_MAC()).hexdigest())


#get our currently used MAC address
def getCurrMAC():
	ret =':'.join(['{:02x}'.format((uuid.getnode() >> i) & 0xff) for i in range(0,8*6,8)][::-1])
	return ret


def resolve_if(interface):
	"""Get the IP address of an interface"""
	addr_type = 2 # 2=AF_INET, 30=AF_INET6
	try:
		addr = netifaces.ifaddresses(interface)[addr_type][0]['addr']
	except Exception as e:
		# failed to resolve
		return None
	return addr

def get_IP():
	"""Try to get this host's active address"""
	interfaces = ["eth0","en0","wlan0"]
	# apppend all interfaces on this host
	#interfaces.append(netifaces.interfaces())
	for interface in interfaces:
		ip = resolve_if(interface)
		if ip!=None: # we have a valid IP
			return ip
	# unknown IP
	return "127.0.0.1"

def get_SA():
	"""get our percieved remote source address"""
	try:
		ret = urllib2.urlopen('http://ip.42.pl/raw').read()
	except Exception as e: # We may be offline
		logger.error("Unable to request ip.42.pl server's view of our IP, we may be offline: %s" % e)
		return "0.0.0.0"
	return ret

def get_temperature():
	"""Return the current temperature sensor, if possible"""
	filename = "/sys/class/thermal/thermal_zone0/temp"
	try:
		f = open(filename, 'r')
		data = f.read()
		return float(data.strip())
	except:
		pass
	return None

#
# Simple statistics to avoid numpy
#
def mean(data):
	"""Return the sample arithmetic mean of data."""
	n = len(data)
	if n < 1:
		raise ValueError('mean requires at least one data point')
	return sum(data)/n # in Python 2 use sum(data)/float(n)


def sumsq(data):
	"""Return sum of square deviations of sequence data."""
	c = mean(data)
	ss = sum((x-c)**2 for x in data)
	return ss


def stdev(data):
	"""Calculates the population standard deviation."""
	n = len(data)
	if n < 2:
		raise ValueError('variance requires at least two data points')
	ss = sumsq(data)
	pvar = ss/n # the population variance
	return pvar**0.5





#!/usr/bin/env python2
from __future__ import unicode_literals, absolute_import, print_function

import pickle
import math

class Data(object):

	def pickle(self, path):
		with open(path, 'wb') as fd:
			pickle.dump(self, fd)

class DistData(Data):

	def __init__(self, source, target, original_distributions,
			distribution_model, samples):

		self._source = source
		self._target = target

		#self._x_values = x_values
		#self._y_original = y_original
		#self._y_model = y_model
		#self._x_hist = x_hist
		#self._y_hist = y_hist
		#self._n_samples = n_samples

		# Array of (iteration, distribution)
		self._original_distributions = original_distributions

		# DistributionModel object
		self._distribution_model = distribution_model

		# Samples
		self._samples = samples

class ValuesData(Data):

	def __init__(self, source, target, real_means, real_variances, real_skews,
			real_kurtosiss, mean_values, variance_values, skew_values,
			kurtosis_values):

		self._source = source
		self._target = target

		# The actual values of the generating distributions
		self._real_means = real_means
		self._real_variances = real_variances
		self._real_skews = real_skews
		self._real_kurtosiss = real_kurtosiss

		# The values at given points in time, in arrays of tuples (index, value)
		self._mean_values = mean_values
		self._variance_values = variance_values
		self._skew_values = skew_values
		self._kurtosis_values = kurtosis_values

class DeltaData(Data):

	def __init__(self, source, target, delta_mean, delta_variance, delta_skew,
			delta_kurtosis):

		self._source = source
		self._target = target

		# Delta values in arrays of tuples (index, value)
		self._delta_mean = delta_mean
		self._delta_variance = delta_variance
		self._delta_skew = delta_skew
		self._delta_kurtosis = delta_kurtosis



import random
import json
from subprocess import Popen, PIPE

import cheesepi as cp

# To be subclassed by explicit measurement tasks
class Task:

	def __init__(self, dao, spec):
		self.dao                = dao
		self.spec               = {}
		self.spec['taskname']   = "Superclass"
		self.spec['inititator'] = cp.utils.get_host_id()
		self.spec['period']     = 0
		self.spec['offset']     = 0
		self.spec['downloaded'] = 1 # bytes, placeholder
		self.spec['uploaded']   = 1 # bytes, placeholder
		# overwrite defauls:
		for s in spec.keys():
			self.spec[s] = spec[s]

		# allow random offsets
		if self.spec['offset']=="rand":
			self.spec['offset'] = random.randint(1, self.spec['period']-1)

	def toDict(self):
		return self.spec

	def toJson(self):
		return json.dumps(self.toDict())

	def execute(self, program):
		result = Popen(program, stdin=PIPE, stdout=PIPE, stderr=PIPE, shell=True)
		result.stdout.flush()
		output = result.stdout.read()
		result.poll() # set return code
		return result.returncode, output

	# this will be overridden by subclasses
	def run(self):
		print "Task not doing anything..."



from __future__ import unicode_literals, absolute_import, print_function

import pickle
import os
import re
import math
from fnmatch import fnmatch

import matplotlib.pyplot as plt
import numpy as np
from statsmodels.sandbox.distributions.extras import pdf_mvsk

from tkinter import Tk, ttk, Toplevel, Menu
import tkinter as tk
import tkFont

import datasets as ds
import mock_ping as mp

class DataEntry(object):

	def __init__(self, parent_frame, grid_col, grid_row, data):

		self._data = data
		self._parent = parent_frame

		self._frame = ttk.Frame(self._parent, borderwidth=2, relief='sunken')
		self._frame.grid(column=grid_col, row=grid_row)

		self._menu = Menu(master=self._parent, tearoff=0)
		self._menu.add_command(label='info', command=self.info_popup)

		self._should_plot = tk.BooleanVar()
		self._should_plot.set(False)

		def menu_popup(event):
			self._menu.post(event.x_root, event.y_root)

		self._chkbox = ttk.Checkbutton(self._frame, text="Plot",
			variable=self._should_plot, onvalue=True)
		self._chkbox.pack()

		self._button = ttk.Button(self._frame,
			text="{} -> {}".format(grid_row, grid_col),
			command=self.info_popup
		)
		self._button.pack()

		self._button.bind('<Button-3>', menu_popup)

	@property
	def data(self):
		return self._data

	@property
	def should_plot(self):
		return self._should_plot.get()

	def info_popup(self):
		top = Toplevel()
		top.title("Info")

		frame = ttk.Frame(top)
		frame.pack()

		sourceLbl = ttk.Label(frame, text="Source")
		targetLbl = ttk.Label(frame, text="Target")

		sourceText = ttk.Label(frame, text=str(self._data._source),
			relief='sunken')
		targetText = ttk.Label(frame, text=str(self._data._target),
			relief='sunken')

		sourceLbl.grid(column=0, row=0)
		targetLbl.grid(column=1, row=0)
		sourceText.grid(column=0, row=1, padx=5, pady=5)
		targetText.grid(column=1, row=1, padx=5, pady=5)

class Plotter(object):

	def __init__(self):
		self._plot_data = []

	def add_dataplot(self, data):
		self._plot_data.append(data)

	def plot(self):
		for data in self._plot_data:
			setup_plot(data)

		plt.show()

	def clear_plots(self):
		self._plot_data = []

def get_data_files(directory):

	files = [f for f in os.listdir(directory) if fnmatch(f, "*.pickle")]

	return files

def unpickle(filepath):
	with open(filepath, 'r') as fd:
		obj = pickle.load(fd)

		# Special case since Numpy distributions can't be pickled, so the
		# arguments are pickled instead and then we recreate the object from
		# the arguments.
		if isinstance(obj, ds.DistData):
			unpacked = []
			for d in obj._original_distributions:
				#print(*d[1][1:])
				nd = mp.GammaDist(*d[1][1:])
				unpacked.append((d[0],nd))
			#print(unpacked)
			obj._original_distributions = unpacked

	return obj

def setup_plot(data):

	if isinstance(data, ds.DistData):
		setup_dist_plot(data)
	elif isinstance(data, ds.DeltaData):
		setup_delta_plot(data)
	elif isinstance(data, ds.ValuesData):
		setup_values_plot(data)

def setup_dist_plot(dist_data):
	fig = plt.figure()
	fig.suptitle("DIST: {}".format(dist_data._source))

	#print(dist_data._samples)

	# Boundaries and x values
	xmax = max(dist_data._samples)
	xmin = min(dist_data._samples)
	xmax = xmax + float(xmax)/10
	xmin = float(xmin)/2
	x_plot = np.linspace(xmin, xmax, xmax-xmin)

	# Histogram
	hist_y, hist_x = np.histogram(dist_data._samples, bins=np.linspace(xmin, xmax,
		xmax-xmin), density=True)

	# Plot histogram
	plt.bar(hist_x[:-1], hist_y,
		width=hist_x[1]-hist_x[0],
		color='green', alpha=0.2, linewidth=0)

	# Model distribution
	model = dist_data._distribution_model
	model_pdf = pdf_mvsk([model._m1, model._new_variance,
			model._skew, model._kurtosis])

	# Distribution y-values
	y_model_plot = np.array([model_pdf(x) for x in x_plot])

	# Distribution plots
	plt.plot(x_plot, y_model_plot, color='r',
		label='Gram-Charlier Expansion (after last iteration)')

	for i, d in enumerate(dist_data._original_distributions):
		#print("{}, {}".format(i, d))
		start_time = d[0]
		dist = d[1].get_dist()
		#print("{} {}".format(start_time, dist))
		y_plot = dist.pdf(x_plot)

		plt.plot(x_plot, y_plot,
			label='Distribution {} (start at iteration {})'.format(i, start_time))

	plt.axvline(x=dist_data._distribution_model._m1, ymin=0, ymax=1, linestyle=':',
		label='Model Mean (after last iteration)')

	plt.title("{}...".format(dist_data._target[:20]), fontdict={'fontsize':10})
	plt.legend(loc='upper right', ncol=1, fontsize=9)

	# Additional Text
	ax = fig.get_axes()
	plt.text(0.70, 0.70, "#samples={}".format(len(dist_data._samples)),
			fontsize=8, transform=ax[0].transAxes)

def setup_delta_plot(delta_data):
	fig = plt.figure()
	fig.suptitle("MV: {}".format(delta_data._source))

	x_plot, y_plot = zip(*delta_data._delta_mean)
	plt.semilogy(x_plot, y_plot, basey=2, linestyle='-', label=r'$\Delta$mean')

	x_plot, y_plot = zip(*delta_data._delta_variance)
	plt.semilogy(x_plot, y_plot, basey=2, linestyle='-', label=r'$\Delta$variance')

	plt.title("{}...".format(delta_data._target[:20]), fontdict={'fontsize':10})
	plt.legend(loc='upper right', ncol=1, fontsize=9)

	plt.grid(True)

	fig = plt.figure()
	fig.suptitle("SK: {}".format(delta_data._source))

	x_plot, y_plot = zip(*delta_data._delta_skew)
	plt.semilogy(x_plot, y_plot, basey=2, linestyle='-', label=r'$\Delta$skew')

	x_plot, y_plot = zip(*delta_data._delta_kurtosis)
	plt.semilogy(x_plot, y_plot, basey=2, linestyle='-', label=r'$\Delta$kurtosis')

	plt.title("{}...".format(delta_data._target[:20]), fontdict={'fontsize':10})
	plt.legend(loc='upper right', ncol=1, fontsize=9)

	plt.grid(True)

def setup_values_plot(values_data):
	fig = plt.figure()
	fig.suptitle("MV: {}".format(values_data._source))

	x_plot, y_plot = zip(*values_data._mean_values)
	plt.plot(x_plot, y_plot, linestyle='-', label=r'mean')

	for i, rm in enumerate(values_data._real_means):
	    plt.axhline(y=rm, xmin=0, xmax=1, linestyle='-.',
		    label="real mean {}".format(i))

	x_plot, y_plot = zip(*values_data._variance_values)
	plt.plot(x_plot, y_plot, linestyle='-', label=r'variance')

	for i, rv in enumerate(values_data._real_variances):
	    plt.axhline(y=rv, xmin=0, xmax=1, linestyle=':',
		    label="real variance {}".format(i))

	plt.title("{}...".format(values_data._target[:20]), fontdict={'fontsize':10})
	plt.legend(loc='upper right', ncol=1, fontsize=9)

	fig = plt.figure()
	fig.suptitle("SK: {}".format(values_data._source))

	x_plot, y_plot = zip(*values_data._skew_values)
	plt.plot(x_plot, y_plot, linestyle='-', label=r'skew')

	for i, rs in enumerate(values_data._real_skews):
	    plt.axhline(y=rs, xmin=0, xmax=1, linestyle='-.',
		    label="real skew {}".format(i))

	x_plot, y_plot = zip(*values_data._kurtosis_values)
	plt.plot(x_plot, y_plot, linestyle='-', label=r'kurtosis')

	for i, rk in enumerate(values_data._real_kurtosiss):
	    plt.axhline(y=rk, xmin=0, xmax=1, linestyle=':',
		    label="real kurtosis {}".format(i))

	plt.title("{}...".format(values_data._target[:20]), fontdict={'fontsize':10})
	plt.legend(loc='upper right', ncol=1, fontsize=9)

class DataExplorer(object):

	def __init__(self, directory):
		self._plotter = Plotter()
		self._data_entries = []

		self._root = Tk()
		self._root.title("Data Explorer")

		self._font = tkFont.Font(root=self._root,family="Helvetica",size=10)
		self._style = ttk.Style()
		self._style.configure('.', font=self._font)

		self._content_frame = ttk.Frame(self._root, width=300)
		self._content_frame.pack()

		# DATA FRAME
		self._data_frame = ttk.Frame(self._content_frame)
		self._data_frame.grid(row=0)

		self._dist_label = ttk.Label(self._data_frame, text='Distributions')
		self._delta_label = ttk.Label(self._data_frame, text='Deltas')
		self._values_label = ttk.Label(self._data_frame, text='Values')

		self._dist_label.grid(column=0, row=0, pady=5)
		self._delta_label.grid(column=1, row=0, pady=5)
		self._values_label.grid(column=2, row=0, pady=5)

		self._dist_frame = ttk.Frame(self._data_frame, borderwidth=5, relief='raised')
		self._dist_frame.grid(column=0, row=1, padx=5)

		self._delta_frame = ttk.Frame(self._data_frame, borderwidth=5, relief='raised')
		self._delta_frame.grid(column=1, row=1, padx=5)

		self._values_frame = ttk.Frame(self._data_frame, borderwidth=5, relief='raised')
		self._values_frame.grid(column=2, row=1, padx=5)

		# CONTROL FRAME
		self._control_frame = ttk.Frame(self._content_frame)
		self._control_frame.grid(row=1, pady=5, padx=5, sticky=(tk.W))

		# LEGEND FRAME
		self._legend_frame = ttk.Frame(self._content_frame)
		self._legend_frame.grid(row=2, pady=5, padx=5)
		legend_labels = {}

		# Set up data
		files = get_data_files(directory)

		for i, f in enumerate(files):
			filepath = os.path.join(directory, f)

			m = re.search('.*(\d+)_(\d+)\.pickle', f)
			# From
			grid_row = int(m.group(1))
			# To
			grid_col = int(m.group(2))

			data_object = unpickle(filepath)

			if grid_row not in legend_labels:
				legend_labels[grid_row] = ttk.Label(self._legend_frame,
					text="{}: {}".format(grid_row, str(data_object._source)))

			if isinstance(data_object, ds.DistData):
				master_frame = self._dist_frame
			elif isinstance(data_object, ds.DeltaData):
				master_frame = self._delta_frame
			elif isinstance(data_object, ds.ValuesData):
				master_frame = self._values_frame

			de = DataEntry(master_frame, grid_col, grid_row, data_object)

			self._data_entries.append(de)

		# Set up control buttons
		plot_button = ttk.Button(self._control_frame,
			text="Plot selected",
			command=self._plot_selected
		)
		plot_button.grid()

		# Set up legend
		legend_head = ttk.Label(self._legend_frame, text="Legend")
		legend_head.grid(row=0, column=0, sticky=(tk.N, tk.W))

		for i, ll in legend_labels.iteritems():
			ll.grid(row=i+1, column=0, sticky=(tk.N, tk.W))

	def _plot_selected(self):
		for de in self._data_entries:
			if de.should_plot:
				self._plotter.add_dataplot(de.data)

		self._plotter.plot()
		self._plotter.clear_plots()

	def start(self):
		self._root.mainloop()

if __name__ == "__main__":
	import argparse

	parser = argparse.ArgumentParser()

	parser.add_argument('--dir', type=str,
		help="The directory where the data is located.")

	args = parser.parse_args()

	if args.dir is not None:
		directory = os.path.abspath(args.dir)
		de = DataExplorer(directory)
		de.start()
	else:
		print("No directory specified.")



from __future__ import unicode_literals, absolute_import

from cheesepi.exceptions import UnsupportedResultType

class Result(object):

	@classmethod
	def fromDict(cls, dct):
		name = dct['task_name']

		from .PingResult import PingResult
		if name == 'ping': return PingResult.fromDict(dct)
		else: raise UnsupportedResultType("Unknown task type '{}'.".format(name))

	def toDict(self):
		raise NotImplementedError("Abstract method 'toDict' not implemented.")

	def get_taskname(self):
		raise NotImplementedError("Abstract method 'taskname' not implemented.")

	def get_target(self):
		raise NotImplementedError("Abstract method 'get_target' not implemented.")



import math
import numpy as np
import matplotlib.pyplot as plt

data = []

def parse_ping_csv(filename):
    data = np.genfromtxt(filename, delimiter=',', skip_header=1, skip_footer=3)
    # Remove any NaN occurence
    data = data[~np.isnan(data)]
    return data

def histplot(data, xmin=0, xmax=1000, step=1):

    y, x = np.histogram(data, bins=np.linspace(xmin, xmax, (xmax-xmin)/step),
            density=True)

    plt.bar(x[:-1], y, width=x[1]-x[0], color='red', alpha=0.5)
    plt.grid(True)
    plt.show()


if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("--file", type=str, help="The csv file to parse")
    parser.add_argument("--max", type=int, default=1000,
            help="The upper limit of the plot range")
    parser.add_argument("--min", type=int, default=0,
            help="The lower limit of the plot range")
    parser.add_argument("--step", type=float, default=1,
            help="The step size of the buckets")

    args = parser.parse_args()

    if args.file is None:
        print("No file given...")
    else:
        data = parse_ping_csv(args.file)
        histplot(data, xmin=args.min, xmax=args.max, step=args.step)



import time
import os

import cheesepi as cp
import Task

logger = cp.config.get_logger(__name__)

class Dummy(Task.Task):

	# construct the process and perform pre-work
	def __init__(self, dao, spec):
		Task.Task.__init__(self, dao, spec)
		self.spec['taskname']    = "dummy"
		if not 'message' in self.spec: self.spec['message'] = "This is not a test"

	# actually perform the measurements, no arguments required
	def run(self):
		logger.info("Dummy: %s @ %f, PID: %d" % (self.spec['message'], time.time(), os.getpid()))


if __name__ == "__main__":
	#general logging here? unable to connect etc
	dao = cp.config.get_dao()

	spec = {}
	dummy_task = Dummy(dao, spec)
	dummy_task.run()




from __future__ import unicode_literals, absolute_import, print_function

import logging
import heapq
import random

from cheesepi.server.storage.mongo import MongoDAO
from .Scheduler import Scheduler

BLIND_SCHEDULE_RATIO=float(1)/float(3)

class PingScheduler(Scheduler):

	log = logging.getLogger("cheesepi.server.scheduling.PingScheduler")

	def __init__(self, uuid):
		self.dao = MongoDAO('localhost', 27017)
		self._uuid = uuid

	def get_random_schedule(self, num=1, ignore_uuids=None):
		"""
		Get a random schedule, does not include self and does not allow for
		duplicates.
		"""
		schedule = []

		if ignore_uuids is None:
			ignore_uuids = [self._uuid]
		else:
			ignore_uuids.append(self._uuid)

		for i in range(0, num):
			entity = self.dao.get_random_entity(ignore_uuids=ignore_uuids)

			if entity is None:
				# We've reached the limit of entities available for scheduling
				break

			schedule.append(entity)
			ignore_uuids.append(entity.get_uuid())

		return schedule

	def get_round_robin_schedule(self, num=1):

		schedule = self.dao.get_sequential_entities(self._uuid, num)

		return schedule

	def get_schedule(self, num=1, ignore_uuids=None):

		if ignore_uuids is None:
			ignore_uuids = [self._uuid]
		else:
			ignore_uuids.append(self._uuid)

		self.log.info("Scheduling for {}".format(self._uuid))
		from pprint import pformat

		#self.log.info("Generating schedule with blind ratio = {}".format(
			#BLIND_SCHEDULE_RATIO))

		# num is 1, we randomize if it will be random or not to achieve full coverage
		if num == 1:
			x = random.uniform(0.0, 1.0)
			if x <= BLIND_SCHEDULE_RATIO:
				non_blind_num = 0
			else:
				non_blind_num = 1
		else:
			non_blind_num = int(num - (num*BLIND_SCHEDULE_RATIO))

		blind_num = num - non_blind_num

		#self.log.info("Non blinds = {}".format(non_blind_num))

		schedule = []
		stats = self.dao.get_all_stats(self._uuid)
		#self.log.info(pformat(stats.toDict()))

		priority_sorted_targets = []

		for s in stats:
			target = s.get_target()
			target_uuid = target.get_uuid()

			if target_uuid not in ignore_uuids:
				#print(pformat(s.toDict()))
				delay = s.get_delay()
				# self.log.info("\ndm1: {}\ndm2: {}\ndm3: {}\ndm4: {}\nsumdm: {}".format(
				# 	delay._dm1, delay._dm2, delay._dm3, delay._dm4,
				# 	delay._dm1 + delay._dm2 + delay._dm3 + delay._dm4)
				# )
				delay_variance = delay.get_variance()
				#print(target)
				#print(delay_variance)
				heapq.heappush(priority_sorted_targets, (-delay_variance, target))

		#print(priority_sorted_targets)

		for i in range(0, min(non_blind_num,len(priority_sorted_targets))):
			target = heapq.heappop(priority_sorted_targets)
			target_uuid = target[1].get_uuid()

			schedule.append(target[1])

			ignore_uuids.append(target_uuid)

		if len(schedule) < non_blind_num:
			# schedule length needs to be filled with more blinds
			blind_num = blind_num + (non_blind_num - len(schedule))

		random_schedule = self.get_random_schedule(blind_num,
				ignore_uuids=ignore_uuids)

		schedule.extend(random_schedule)
		#for i in range(0, blind_num):
			##self.log.info("Adding blind to schedule")
			#entity = self.dao.get_random_entity(ignore_uuid=self._uuid)
			#schedule.append(entity)

		return schedule


if __name__ == "__main__":
	import argparse
	import cheesepi.server.utils as utils

	utils.init_logging()

	parser = argparse.ArgumentParser()
	parser.add_argument('--id', type=str, default=None)
	parser.add_argument('--num', type=int, default=1)

	args = parser.parse_args()

	ps = PingScheduler(args.id)

	schedule = ps.get_schedule(num=args.num)
	if len(schedule) == 0:
		print("Nothing scheduled")
	else:
		for target in schedule:
			print(target.get_uuid())



import time
import os
from subprocess import Popen, PIPE

import cheesepi as cp
import Task

logger = cp.config.get_logger(__name__)

class VoIP(Task.Task):

	# construct the process and perform pre-work
	def __init__(self, dao, spec):
		Task.Task.__init__(self, dao, spec)
		self.spec['taskname']    = "voip"
		if not 'peer'    in self.spec: self.spec['peer']    = "somehost.com"

	# actually perform the measurements, no arguments required
	def run(self):
		logger.info("Ringing: %s @ %f, PID: %d" % (self.spec['landmark'], time.time(), os.getpid()))
		self.measure()

	# measure and record funtion
	def measure(self):
		start_time = cp.utils.now()
		#op_output = self.perform(self.spec['landmark'], self.spec['ping_count'], self.spec['packet_size'])
		end_time = cp.utils.now()
		#logger.debug(op_output)
		#parsed_output = self.parse_output(op_output, self.spec['landmark'])
		#self.dao.write_op(self.spec['taskname'], parsed_output)

	#ping function
	def perform(self, landmark, ping_count, packet_size):
		pass

	#read the data from ping and reformat for database entry
	def parse_output(self, data, peer):
		pass


if __name__ == "__main__":
	dao = cp.config.get_dao()

	spec = {}
	voip_task = VoIP(dao, spec)
	voip_task.run()
	print voip_task.spec




from __future__ import unicode_literals, absolute_import, print_function

import logging
import sys

from zope.interface import provider
from twisted.logger import ILogObserver, formatEvent

@provider(ILogObserver)
class PrintingObserver:
	def __call__(self, event):
		print("[{level}] {text}".format(
			level=event['log_level'].name.upper(),
			text=formatEvent(event)))

def init_logging(level=logging.INFO, stdout=True):
	# Python Logging
	logging.basicConfig()
	logging.root.setLevel(logging.INFO)

	# Write logs to stdout
	if stdout:
		out_handler = logging.StreamHandler(sys.stdout)
		formatter = logging.Formatter('[%(levelname)s][%(name)s]: %(message)s')
		out_handler.setFormatter(formatter)
		logging.root.addHandler(out_handler)

### Script entrypoints

def start_control_server():
	import argparse

	from twisted.internet import reactor
	from twisted.logger import Logger, globalLogPublisher, STDLibLogObserver

	from cheesepi.server.control import (CheeseRPCServerFactory,
	                                     CheeseRPCServer)
	from cheesepi.server.storage.mongo import MongoDAO

	# Argument parsing
	parser = argparse.ArgumentParser()
	parser.add_argument('--port', type=int, default=18080,
	                    help='Port to listen on')
	args = parser.parse_args()

	init_logging()

	# Make twisted logging write to pythons logging module
	globalLogPublisher.addObserver(STDLibLogObserver(name="cheesepi.server.control"))

	# Use twisted logger when in twisted
	log = Logger()

	# Logging
	#log = Logger()
	#globalLogPublisher.addObserver(PrintingObserver())

	#dao = MongoDAO()
	dao = MongoDAO('localhost', 27017)
	control_server = CheeseRPCServer(dao).getStreamFactory(CheeseRPCServerFactory)

	reactor.listenTCP(args.port, control_server)
	log.info("Starting control server on port %d..." % args.port)
	reactor.run()

def start_upload_server():
	import argparse

	from twisted.internet import reactor
	from twisted.logger import Logger, globalLogPublisher, STDLibLogObserver
	from twisted.web.server import Site
	from twisted.web.resource import Resource

	from cheesepi.server.upload import UploadHandler

	# Argument parsing
	parser = argparse.ArgumentParser()
	parser.add_argument('--port', type=int, default=18090,
	                    help='Port to listen on')
	args = parser.parse_args()

	init_logging()

	# Make twisted logging write to pythons logging module
	globalLogPublisher.addObserver(STDLibLogObserver(name="cheesepi.server.upload"))

	# Use twisted logger when in twisted
	log = Logger()

	root = Resource()
	root.putChild("upload", UploadHandler())
	upload_server = Site(root)

	reactor.listenTCP(args.port, upload_server)
	log.info("Starting upload server on port %d..." % args.port)
	reactor.run()
